'use strict';

var Vue = require('vue');
Vue = 'default' in Vue ? Vue['default'] : Vue;
var minimatch = require('minimatch');

var babelHelpers = {};

babelHelpers.createClass = (function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
})();

babelHelpers.classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};
var isDev = typeof __DEV__ !== 'undefined' && __DEV__;
if (isDev) {
  Vue.config.debug = true;
}

var Router = (function () {
  function Router() {
    babelHelpers.classCallCheck(this, Router);

    this.routes = [];
    this.Vue = Vue;
    this.includeQuery = false;
  }

  babelHelpers.createClass(Router, [{
    key: 'use',
    value: function use(name) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      this.Vue.use(name, options);
      return this;
    }
  }, {
    key: 'set',
    value: function set(name, value) {
      this[name] = value;
      return this;
    }
  }, {
    key: 'reg',
    value: function reg(name, route) {
      if (isDev && (!route || !route.view)) {
        throw new ReferenceError('[VSR] Could not reference route view');
      }
      if ('default' in route.view) {
        route.view = route.view['default'];
      }
      this.routes.push({
        pattern: name,
        view: route.view
      });
      return this;
    }
  }, {
    key: 'init',
    value: function init() {
      var bindTo = arguments.length <= 0 || arguments[0] === undefined ? 'body' : arguments[0];

      this.bindTo = bindTo;
      var fullPath = this.includeQuery ? location.pathname + location.search : location.pathname;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this.routes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var route = _step.value;
          var pattern = route.pattern;
          var view = route.view;

          var isMiniMatch = typeof pattern === 'string' && pattern === fullPath;
          if (isMiniMatch || pattern.test(fullPath)) {
            view.el = this.bindTo;
            // we initial its kids before itself
            // for the scope matter
            // you cannot initial a parent instance
            // then initial its child element
            // same, the parent instance would not own the control of its children
            this.initKids(view.kids);
            new this.Vue(view);
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator['return']) {
            _iterator['return']();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return this;
    }
  }, {
    key: 'initKids',
    value: function initKids(kids) {
      var _this = this;

      if (kids && Array.isArray(kids)) {
        kids.forEach(function (kid) {
          _this.initKids(kid.kids);
          new _this.Vue(kid);
        });
      }
    }
  }]);
  return Router;
})();

var index = new Router();

module.exports = index;